=begin
    yagettext.rb - Yet Another GetText
    Copyright (C) 2001  Masahiro Sakai

        Masahiro Sakai                  <s01397ms@sfc.keio.ac.jp>

    You can redistribute this file and/or modify it under the same term
    of Ruby.  License of Ruby is included with Ruby distribution in
    the file "README".
=end

require 'mo.rb'

module YAGetText

  class TextDomain
    attr_reader :name

    def initialize(name, dirname = Dir.pwd)
      @name     = name
      @dirname  = dirname
      @mofiles  = Hash.new
    end

    def bind(dirname = Dir.pwd)
      @dirname = dirname
      @mofiles.clear
    end

    def gettext(msgid, category = 'LC_MESSAGES', locale = get_locale(category))
      get_mo(category, locale)[msgid] or msgid
    end

    def [](msgid)
      gettext(msgid)
    end

    private

    Null = MOFile.new

    def get_locale(category)
      ENV['LANG'] or ENV['LC_ALL'] or ENV[category] or 'C'
    end

    def locale_candidates(locale)
      locale = String(locale)
      result = []

      matched = /^([A-Za-z]+)(_[A-Za-z]+)?(\.\w+)?((?:@[^@]*)*)$/.match(locale)
      if matched.size > 1
        matched = matched.to_a
        matched.shift # remove original string
        matched.delete(nil)
        matched.delete('')
        while not matched.empty?
          result.push(matched.join(''))
          matched.pop
        end
      else
        result.push(locale)
      end

      result
    end

    def get_mo(category, locale)
      category = String(category)
      locale   = String(locale)
      key      = [category, locale]

      if @mofiles.has_key? key
        @mofiles[key]
      else
        locale_candidates(locale).each{|locale|
          fname = File.join(@dirname, locale, category, @name + '.mo')
          if File.exist? fname
            return @mofiles[key] = MOFile.open(fname)
          end
        }
        @mofiles[key] = Null
      end
    end    
  end

  ###########################################################################

  @@domains = {}
  @@default_domain = nil  

  def self.__get_textdomain_object(name)
    @@domains[name] = TextDomain.new(name) unless @@domains.has_key? name
    @@domains[name]
  end

  def bindtextdomain(name, *args)
    YAGetText.__get_textdomain_object(name).bind(*args)
    nil
  end
  module_function :bindtextdomain

  def dgettext(name, str, *args)
    if @@domains.has_key? name
      @@domains[name].gettext(str, *args)
    else
      str
    end
  end
  module_function :dgettext

  def self.textdomain(name = nil)
    if name
      @@default_domain = YAGetText.__get_textdomain_object(name)
      name
    elsif @@default_domain
      @@default_domain.name
    else
      nil
    end
  end

  def textdomain(name = nil)
    if name
      @__text_domain = YAGetText.__get_textdomain_object(name)
      name
    elsif (defined? @__text_domain) and @__text_domain
      @__text_domain.name
    else
      YAGetText.textdomain(name)
    end
  end

  def self.gettext(str, *args)
    if @@default_domain
      @@default_domain.gettext(str, *args)
    else
      str
    end
  end

  def gettext(str, *args)
    if (defined? @__text_domain) and @__text_domain
      @__text_domain.gettext(str, *args)
    else
      YAGetText.gettext(str, *args)
    end
  end

  alias _ gettext
end


# Test
if $0 == __FILE__
  include YAGetText
  bindtextdomain('libc', '/usr/share/locale')
  textdomain('libc')
  puts _("Terminated")
end
